import logging

from pprint import pprint

_LOGGER = logging.getLogger(__name__)

# Wireless dimmer
# 1002 Move to level 255
# 2002 Move up
# 3002 Move down
# 4002 Move to level 0


class DeconzSensor:
    """
    """
    def __init__(self, sensor):
        """
        """
        self.state = None
        self.name = sensor['name']
        self.type = sensor['type']
        self.modelid = sensor['modelid']
        self.swversion = sensor['swversion']
        self.uniqueid = sensor['uniqueid']
        self.manufacturer = sensor['manufacturername']

        self.battery = sensor['config']['battery']
        self.reachable = sensor['config']['reachable']

        self.callback = None
    
    def update_config(self, config):
        """
        """
        self.battery = config['battery']
        self.reachable = config['reachable']
    
    def update_state(self, state):
        """
        """
        raise NotImplementedError

    def update(self, event):
        """
        """
        if 'state' in event:
            self.update_state(event['state'])
        if 'config' in event:
            self.update_config(event['config'])
        if self.callback:
            self.callback()
        pprint(self.__dict__)

    def as_dict(self):
        """Callback for __dict__.
        """
        cdict = self.__dict__.copy()
        if 'callback' in cdict:
            del cdict['callback']
        return cdict


class ZHAPresence(DeconzSensor):
    """
    """
    def __init__(self, sensor):
        """
        """
        super().__init__(sensor)
        self.state = False
        self.dark = sensor.get('dark')

    def update_state(self, state):
        """
        """
        self.state = state['presence']
        self.dark = state.get('dark')

    @property
    def is_tripped(self):
        """Event is tripped now.
        """
        return self.state


class ZHASwitch(DeconzSensor):
    """
    """
    def __init__(self, sensor):
        """
        """
        super().__init__(sensor)
        self.state = None

    def update_state(self, state):
        """
        """
        self.state = state['buttonevent']



import logging

from pprint import pprint

_LOGGER = logging.getLogger(__name__)


class DeconzLight(object):
    """
    """
    def __init__(self, light, set_state):
        """
        """
        print(light)
        self.state = light['state']['on']
        self.name = light['name']
        self.type = light['type']
        self.modelid = light['modelid']
        self.swversion = light['swversion']
        self.uniqueid = light['uniqueid']
        self.manufacturer = light['manufacturername']

        self.hascolor = light['hascolor']
        self.brightness = light['state']['bri']
        self.reachable = light['state']['reachable']

        self.set_state = set_state
        self.callback = None

    def update_state(self, state):
        """
        """
        if 'on' in state:
            self.state = state['on']
        if 'bri' in state:
            self.brightness = state['bri']

    def update(self, event):
        """
        """
        if 'state' in event:
            self.update_state(event['state'])
        if self.callback:
            self.callback()
        pprint(self.__dict__)

    def as_dict(self):
        """Callback for __dict__.
        """
        cdict = self.__dict__.copy()
        if 'set_state' in cdict:
            del cdict['set_state']
        if 'callback' in cdict:
            del cdict['callback']
        return cdict



import aiohttp
import asyncio
import json
import logging

from .light import DeconzLight
from .sensor import (ZHASwitch, ZHAPresence)
from .utils import request
from .websocket import WSClient

from pprint import pprint

_LOGGER = logging.getLogger(__name__)


class DeconzSession:
    """
    """

    def __init__(self, loop, host, port, api_key, **kwargs):
        """
        """
        self.lights = {}
        self.sensors = {}
        self.loop = loop
        self.host = host
        self.__session = aiohttp.ClientSession(loop=loop)
        self.__api_url = 'http://%s:%d/api/%s' % (host, port, api_key)
        self.__websocket = None
        self.__websocket_port = None

    def start(self):
        self.__websocket = WSClient(
            self.loop, self.host, self.__websocket_port, self.event_handler)

    @asyncio.coroutine
    def close(self):
        """
        """
        yield from self.__session.close()
        if self.__websocket:
            yield from self.__websocket.stop()

    @asyncio.coroutine
    def populate_config(self):
        config = yield from self.get_state_async('/config')
        self.__websocket_port = config['websocketport']
        pprint(config)
        # 'swversion': '2.4.82'
        # zigbeechannel

    @asyncio.coroutine
    def populate_lights(self):
        lights = yield from self.get_state_async('/lights')
        for light_id, light in lights.items():
            self.lights[light_id] = DeconzLight(light, self.put_state_async)
            _LOGGER.debug('Sensors: %s', self.lights[light_id].__dict__)

    @asyncio.coroutine
    def populate_sensors(self):
        sensors = yield from self.get_state_async('/sensors')
        for sensor_id, sensor in sensors.items():
            if sensor['type'] == 'ZHASwitch':
                self.sensors[sensor_id] = ZHASwitch(sensor) 
            elif sensor['type'] == 'ZHAPresence':
                self.sensors[sensor_id] = ZHAPresence(sensor)
            _LOGGER.debug('Sensors: %s', self.sensors[sensor_id].__dict__)

    @asyncio.coroutine
    def put_state_async(self, field, data):
        """
        """
        session = self.__session.put
        url = self.__api_url + field
        jsondata = json.dumps(data)
        response_dict = yield from request(session, url, data=jsondata)
        return response_dict

    @asyncio.coroutine
    def get_state_async(self, field):
        """
        """
        session = self.__session.get
        url = self.__api_url + field
        response_dict = yield from request(session, url)
        return response_dict
    
    def event_handler(self, event):
        if event['r'] == 'sensors' and event['id'] in self.sensors:
            self.sensors[event['id']].update(event)
        elif event['r'] == 'lights' and event['id'] in self.lights:
            self.lights[event['id']].update(event)
        else:
            # new device, register
            print('not accepted', event)



# https://jeffknupp.com/blog/2013/08/16/open-sourcing-a-python-project-the-right-way/
# http://peterdowns.com/posts/first-time-with-pypi.html
# pip install /home/pi/pydeconz/ --target /home/pi/home-assistant/venv/lib/python3.4/site-packages/ --upgrade
# Upload to PyPI Live
# python setup.py sdist upload -r pypi

from setuptools import setup

setup(
  name='pydeconz',
  packages=['pydeconz'],
  version='0',
  description='A python library for communicating with deconz REST-API from dresden elektronik',
  author='Robert Svensson',
  author_email='Kane610@users.noreply.github.com',
  license='MIT',
  url='https://github.com/Kane610/pydeconz',
  download_url='https://github.com/Kane610/pydeconz/archive/v1.tar.gz',
  install_requires=['aiohttp==2.2.5'],
  keywords=['deconz', 'zigbee', 'homeassistant'],
  classifiers=[
    'Natural Language :: English',
    'Programming Language :: Python :: 3',
  ],
)



import asyncio
import json


class WSClient(asyncio.Protocol):
    """Websocket transport, session handling, message generation.
    """

    def __init__(self, loop, host, port, callback):
        """
        """
        self.loop = loop
        self.transport = None
        self.host = host
        self.port = port
        self.callback = callback
        self.setup_response = True
        conn = loop.create_connection(lambda: self, host, port)
        task = loop.create_task(conn)

    def stop(self):
        """
        """
        if self.transport:
            self.transport.close()

    def connection_made(self, transport):
        """
        """
        import hashlib
        import os
        from base64 import encodestring as base64encode
        randomness = os.urandom(16)
        key = base64encode(randomness).decode('utf-8').strip()
        self.transport = transport
        message = "GET / HTTP/1.1\r\n"
        message += "Host: " + self.host + ':' + str(self.port) + '\r\n'
        message += "User-Agent: Python/3.5 websockets/3.4\r\n"
        message += "Upgrade: Websocket\r\n"
        message += "Connection: Upgrade\r\n"
        message += "Sec-WebSocket-Key: " + key + "\r\n"
        message += "Sec-WebSocket-Version: 13\r\n"
        message += "\r\n"
        print(message)
        self.transport.write(message.encode())

    def data_received(self, data):
        """
        """
        if self.setup_response:
            self.setup_response = False
            return

        head = ord(data[1:2])
        if head <= 125:
            'No extra payload information.'
            payload_start = 2
        elif head == 126:
            'Payload information are an extra 2 bytes.'
            payload_start = 4
        elif head == 127:
            'Payload information are an extra 6 bytes.'
            payload_start = 8

        event = json.loads(data[payload_start:].decode())
        self.callback(event)


    def connection_lost(self, exc):
        """
        """
        print('connection_lost', exc)
        pass
